﻿Type=Class
Version=3.78
@EndOfDesignText@
'Class module
Sub Class_Globals
    Type  TGridDataRowColumn(ColID As Int, Text As String, Color As Int, TextColor As Int, FontSize As Int, Formating As Int)
    Type  TGridDataRow(RowID As Int, Columns As List)  '  Columns is a List of TGridDataRowColumn
	
    Type  TGridHeaderColumn(Fixed As Boolean, Name As String, Width As Int, Color As Int, TextColor As Int, FontSize As Int, Formating As Int)



	Private mScrollTimer                 As Timer

	Private mInScrollChange              As Boolean = False
	Private mLastScrollChange(2)         As Int     = Array As Int(0, 0)
	
	Private mSnapToGridPositionsH()      As Int
	Private mSnapToGridPositionsV()      As Int
	
	Private mGridSize                    As Int    = 2dip
	Private mGridSizeColor               As Int    = Colors.Black
	
	Private mGridSpacing                 As Int    = 0dip	
	Private mGridSpacingColor            As Int    = Colors.LightGray
	
    Private mTop                         As Int    = 0dip
	Private mLeft                        As Int    = 0dip
	Private mWidth                       As Int    = 100%x
	Private mHeight                      As Int    = 100%y
	
    Private mRows                        As Int    = 0
	Private mCols                        As Int    = 0
	
	Private mFixedCols                   As Int    = 0
	Private mFixedColsWidth              As Int    = 0
	
	Private mFixedHeight                 As Int    = 0
	Private mFixedDataHeight             As Int    = 0
	
	Private mHeaderColsWidth             As Int    = 0
	
	Private mOverlayTouchX               As Int
	Private mOverlayTouchY               As Int
	
	Private mHeaderColumns               As List        '  Each Entry Defines a Header Column
	Private mDataRows                    As List        '  Each Entry is a Row (TGridDataRow) which contains a List of Columns (TGridDataRowColumn)

	Private mGrid_Main_Panel             As Panel
	    Private mGrid_FixedColumns_Panel As Panel
	    Private mGrid_FixedHeader_HSV    As HorizontalScrollView
	    Private mGrid_FixedColumns_VSV   As ScrollView	
	    Private mGrid_Body_SV2D          As ScrollView2D
		Private mGrid_Overlay_Panel      As Panel
End Sub

#Region GeneralRoutines
Sub SetGridSize(Size As Int)                      : mGridSize         = Size                :    SetHeaderHeight(mFixedDataHeight)     :      End Sub
Sub SetGridSizeColor(Color As Int)                : mGridSizeColor    = Color               :                                                 End Sub

Sub SetGridSpacing(Spacing As Int)                : mGridSpacing      = Spacing             :    SetHeaderHeight(mFixedDataHeight)     :      End Sub
Sub SetGridSpacingColor(Color As Int)             : mGridSpacingColor = Color               :                                                 End Sub

Sub GridSize()    As Int                          : Return mGridSize                        :                                                 End Sub
Sub GridSizeColor() As Int                        : Return mGridSizeColor                   :                                                 End Sub

Sub GridSpacing() As Int                          : Return mGridSpacing                     :                                                 End Sub
Sub GridSpacingColor() As Int                     : Return mGridSpacingColor                :                                                 End Sub

Sub SetHeaderHeight(Height As Int)	   
	mFixedHeight     = Height + ((mGridSpacing + mGridSize) * 2)
	mFixedDataHeight = Height
End Sub
#End Region


#Region CreateHeaderColumn
Sub CreateHeaderColumn(Fixed As Boolean, Name As String, Width As Int, Color As Int, TextColor As Int, FontSize As Int, Formating As Int) As TGridHeaderColumn
    Dim HeaderColumn As TGridHeaderColumn
	
    HeaderColumn.Initialize
	HeaderColumn.Fixed       = Fixed
    HeaderColumn.Name        = Name
	HeaderColumn.Width       = Width
	HeaderColumn.Color       = Color
	HeaderColumn.TextColor   = TextColor
	HeaderColumn.FontSize    = FontSize
	HeaderColumn.Formating   = Formating

    Return HeaderColumn	
End Sub
#End Region


#Region Initialize(Act As Activity, Left As Int, Top As Int, Width As Int, Height As Int)
Public Sub Initialize(Act As Activity, Left As Int, Top As Int, Width As Int, Height As Int)

    mScrollTimer.Initialize("cGrid_ScrollTimer", 350)
	mScrollTimer.Enabled = False
	
	mLeft   = Left
    mTop    = Top	
	mWidth  = Width
	mHeight = Height

    mGrid_Main_Panel.Initialize("cGrid_Main_Panel")
	
	Act.AddView(mGrid_Main_Panel, mLeft, mTop, mWidth, mHeight)
		
	mGrid_Main_Panel.Left       = mLeft
    mGrid_Main_Panel.Top        = mTop	
	mGrid_Main_Panel.Width      = mWidth
	mGrid_Main_Panel.Height     = mHeight

	mGrid_FixedColumns_Panel.Initialize("cGrid_FixedHeader_Panel")

	mGrid_FixedHeader_HSV.Initialize(mWidth, "cGrid_FixedHeader_HSV")
	mGrid_FixedColumns_VSV.Initialize2(mHeight - mWidth, "cGrid_FixedColumns_VSV")	
	
	mGrid_Body_SV2D.Initialize(10dip, 10dip, "cGrid_Body_SV2D")
	
	mGrid_Overlay_Panel.Initialize("cGrid_Overlay_Panel")
    mGrid_Overlay_Panel.Color = Colors.ARGB(20, 0, 0, 32)

	mGrid_Main_Panel.AddView(mGrid_FixedColumns_Panel, 0, 0, 0, 0)	
	mGrid_Main_Panel.AddView(mGrid_FixedHeader_HSV,    0, 0, 0, 0)
	mGrid_Main_Panel.AddView(mGrid_FixedColumns_VSV,   0, 0, 0, 0)	
	mGrid_Main_Panel.AddView(mGrid_Body_SV2D,          0, 0, 0, 0)
	
	mGrid_Main_Panel.AddView(mGrid_Overlay_Panel,      mLeft, mTop, mWidth, mHeight)
	
	

    mHeaderColumns.Initialize	   	
	mDataRows.Initialize
End Sub
#End Region

#Region ShowGrid
Public Sub ShowGrid
       mFixedCols                      = CalcFixedColumns
	   	   
	   mFixedColsWidth                 = FixedColumnsWidth	   
	   mHeaderColsWidth                = HeaderColumnsWidth

	   mGrid_FixedColumns_Panel.Top    = 0
       mGrid_FixedColumns_Panel.Left   = 0
	   mGrid_FixedColumns_Panel.Width  = mFixedColsWidth
	   mGrid_FixedColumns_Panel.Height = mGrid_Main_Panel.Height - mFixedHeight

       mGrid_FixedHeader_HSV.Top       = 0
	   mGrid_FixedHeader_HSV.Left      = mFixedColsWidth  - mGridSize
	   mGrid_FixedHeader_HSV.Width     = mHeaderColsWidth
	   mGrid_FixedHeader_HSV.Height    = mFixedHeight
	   
	   mGrid_Body_SV2D.Top             = mFixedHeight
	   mGrid_Body_SV2D.Left            = mFixedColsWidth         - mGridSize
	   mGrid_Body_SV2D.Width           = mGrid_Main_Panel.Width  - mFixedColsWidth
	   mGrid_Body_SV2D.Height          = mGrid_Main_Panel.Height - mFixedHeight
	   
	   
	   Dim  CurCol             As Int
	   Dim  CurRow             As Int
	   
	   Dim  CurFixedLeft       As Int = 0
	   Dim  CurHeaderLeft      As Int = 0
	   Dim  CurGridBodyLeft    As Int = 0
	   Dim  CurOffset          As Int
		  
	   If mFixedCols > 0 Then
		  CurHeaderLeft = 0
       End If


       mGrid_FixedColumns_Panel.RemoveAllViews
       mGrid_FixedHeader_HSV.Panel.RemoveAllViews

       mGrid_FixedColumns_VSV.Panel.RemoveAllViews	   
	   mGrid_Body_SV2D.Panel.RemoveAllViews
	   
	   Dim 	SnapToGridPositionsH(mHeaderColumns.Size)   As Int

    
	   For CurCol = 0 To mHeaderColumns.Size-1
	       Dim Column      As TGridHeaderColumn = mHeaderColumns.Get(CurCol)
		   
           Dim ColumnFrame As Panel	   
           Dim ColumnData  As Label

           ColumnFrame.Initialize("cGrid_Frame")
		   ColumnFrame.Tag        = CurCol
		   ColumnFrame.Color      = mGridSpacingColor

           ColumnData.Initialize(Column.Name)
		   ColumnData.Tag        = CurCol
		   ColumnData.Text       = Column.Name
		   ColumnData.Color      = Column.Color
		   ColumnData.TextColor  = Column.TextColor
		   ColumnData.TextSize   = Column.FontSize
		   ColumnData.Gravity    = Column.Formating
		   
		   ColumnFrame.AddView(ColumnData, mGridSpacing, mGridSpacing, Column.Width, mFixedDataHeight)
		   		   	   
	       #If _Debug Then 
		       Log("CurFixedLeft:"&CurFixedLeft&"  CurHeaderLeft:"&CurHeaderLeft&"  Width:"&Column.Width)
           #end if					   

	       If CurCol < mFixedCols Then		   
	          mGrid_FixedColumns_Panel.AddView(ColumnFrame, CurFixedLeft+mGridSize, mGridSize, Column.Width+mGridSpacing+mGridSpacing, mFixedDataHeight+mGridSpacing+mGridSpacing)

			  CurFixedLeft = CurFixedLeft + Column.Width + mGridSpacing + mGridSpacing + mGridSize
		   Else		
              SnapToGridPositionsH(CurCol) = CurHeaderLeft
		   
	          mGrid_FixedHeader_HSV.Panel.AddView(ColumnFrame, CurHeaderLeft+mGridSize, mGridSize, Column.Width+mGridSpacing+mGridSpacing, mFixedDataHeight+mGridSpacing+mGridSpacing)
			  
			  CurHeaderLeft = CurHeaderLeft + Column.Width + mGridSpacing + mGridSpacing + mGridSize		
		   End If		   
	   Next
	   
       mSnapToGridPositionsH              = SnapToGridPositionsH

	   mGrid_FixedHeader_HSV.Panel.Width  = CurHeaderLeft + mGridSize
	   mGrid_FixedHeader_HSV.Panel.Color  = mGridSizeColor
	   mGrid_FixedHeader_HSV.Width        = 100%x - mFixedColsWidth

	   mGrid_FixedColumns_Panel.Color     = mGridSizeColor
	   mGrid_FixedColumns_Panel.Height    = mFixedHeight
	   mGrid_FixedColumns_Panel.Width     = mFixedColsWidth
	   
'	   mGrid_FixedColumns_VSV.Panel.Color = mGridSizeColor
'	   mGrid_FixedColumns_VSV.Height      = mFixedHeight


	   mGrid_FixedColumns_VSV.Top      = mFixedHeight - mGridSize
       mGrid_FixedColumns_VSV.Left     = 0
	   mGrid_FixedColumns_VSV.Width    = mFixedColsWidth + mGridSize
	   mGrid_FixedColumns_VSV.Height   = mGrid_Main_Panel.Height - mFixedHeight
	   
	   
	   Dim 	SnapToGridPositionsV(mDataRows.Size)   As Int
	   
	   For CurRow = 0 To mDataRows.Size-1
	       Log("CurRow:" &CurRow)

	       CurFixedLeft      = 0
	       CurGridBodyLeft   = 0
		   
	       CurOffset = (CurRow * (mFixedHeight - mGridSize))
		   
           SnapToGridPositionsV(CurRow) = CurOffset
		   
		   
	       For CurCol = 0 To mHeaderColumns.Size-1
	           Dim Column  As TGridHeaderColumn = mHeaderColumns.Get(CurCol)

               Dim ColumnFrame As Panel	   
               Dim ColumnData  As Label

               ColumnFrame.Initialize("cGrid_Frame")
    		   ColumnFrame.Tag        = CurCol
	    	   ColumnFrame.Color      = mGridSpacingColor

               ColumnData.Initialize(Column.Name)
    		   ColumnData.Tag        = CurCol
	    	   ColumnData.Text       = CurRow &"."&CurCol
			   
			   If Column.Fixed Then 
		          ColumnData.Color      = Column.Color
    		      ColumnData.TextColor  = Column.TextColor			   
			   Else		  
		          ColumnData.Color      = Colors.White
    		      ColumnData.TextColor  = Colors.Blue
			   End If
			   
	    	   ColumnData.TextSize   = Column.FontSize
		       ColumnData.Gravity    = Column.Formating
		   
    		   ColumnFrame.AddView(ColumnData, mGridSpacing, mGridSpacing, Column.Width, mFixedDataHeight)
			   
    	       #If _Debug Then 
	    	       Log("CurFixedLeft:"&CurFixedLeft&"  CurGridBodyLeft:"&CurGridBodyLeft&"  Width:"&Column.Width)
               #end if					   
			   
	           If CurCol < mFixedCols Then		   
			      'Log("Adding FixedCols - CurrOffSet:" &CurOffset &"  Left:" &CurFixedLeft)
	              mGrid_FixedColumns_VSV.Panel.AddView(ColumnFrame, CurFixedLeft+mGridSize, CurOffset, Column.Width+mGridSpacing+mGridSpacing, mFixedDataHeight+mGridSpacing+mGridSpacing)

			      CurFixedLeft = CurFixedLeft + Column.Width + mGridSpacing + mGridSpacing + mGridSize
    	       Else		
			      'Log("Adding MoveableCols - CurrOffSet:" &CurOffset &"  Left:" &CurGridBodyLeft)

                  mGrid_Body_SV2D.Panel.AddView(ColumnFrame, CurGridBodyLeft+mGridSize, CurOffset, Column.Width+mGridSpacing+mGridSpacing, mFixedDataHeight+mGridSpacing+mGridSpacing)
			  
       			  CurGridBodyLeft = CurGridBodyLeft + Column.Width + mGridSpacing + mGridSpacing + mGridSize		
	    	   End If		   
    	   Next
	   Next
	   
       mSnapToGridPositionsV        = SnapToGridPositionsV
	   
	   
	   mGrid_Body_SV2D.Color        = Colors.Yellow
       mGrid_Body_SV2D.Panel.Color  = mGridSizeColor
	   
	   mGrid_Body_SV2D.Top          = mFixedHeight
       mGrid_Body_SV2D.Left         = mFixedColsWidth - mGridSize
	   mGrid_Body_SV2D.Height       = 100%y - mFixedHeight
       mGrid_Body_SV2D.Width        = 100%x - mFixedColsWidth
	   
	   mGrid_Body_SV2D.Panel.Height  = CurOffset
       mGrid_Body_SV2D.Panel.Width   = CurGridBodyLeft + mGridSize
	   
       mGrid_Body_SV2D.FadingEdges(False)
	   mGrid_Body_SV2D.ScrollbarsVisibility(False, False)
	      
	   
	   mGrid_FixedColumns_VSV.Panel.Color  = mGridSizeColor
	  'mGrid_FixedColumns_VSV.Panel.Height = CurOffset + mGridSize
	   mGrid_FixedColumns_VSV.Top      = mFixedHeight
       mGrid_FixedColumns_VSV.Left     = 0
	   mGrid_FixedColumns_VSV.Width    = mFixedColsWidth + mGridSize
	   mGrid_FixedColumns_VSV.Height   = 100%y - mFixedHeight
'	   mGrid_FixedColumns_VSV.Height       = mGrid_Main_Panel.Height - mFixedDataHeight
	  
	   mGrid_FixedColumns_VSV.Panel.Height = CurOffset	   	   
	   
       Dim r  As Reflector
	   Dim r2 As Reflector
	   
       r.Target = mGrid_FixedColumns_VSV
       r.RunMethod2("setVerticalScrollBarEnabled"  , False, "java.lang.boolean")	   
	   
       r2.Target = mGrid_FixedHeader_HSV
       r2.RunMethod2("setHorizontalScrollBarEnabled", False, "java.lang.boolean")	   	   
End Sub
#End Region

#Region GridHeader(Height As Int, Columns() As TGridHeaderColumn)
Public Sub GridHeader(Height As Int, Columns() As TGridHeaderColumn)

       Dim CurCol As Int 
	   
		  
	   mHeaderColumns.Clear	   
	   mDataRows.Clear                
	   
	   For CurCol = 0 To Columns.Length-1
	       Dim Column As TGridHeaderColumn = Columns(CurCol)
		   
		   mHeaderColumns.Add(Column)
		   
		   '----------------------------------------------------------------
		   '  The code below is ONLY here until we make a Add Data Row
		   '      Currently we are just using the Column Data as a Data Row
		   '----------------------------------------------------------------
           #If _Debug Then 
		       mDataRows.Add(Columns(CurCol).Name)
		   #End If 		   
	   Next

	   
	   SetHeaderHeight(Height)
	   
   
	   ShowGrid
End Sub
#End Region

#Region AddGridColumn(Column As TGridHeaderColumn)
Sub AddGridColumn(Column As TGridHeaderColumn) As Boolean

    If mHeaderColumns.Size > 0 Then
       Dim LastCol As TGridHeaderColumn = mHeaderColumns.Get(mHeaderColumns.Size-1)
	   
	   If LastCol.Fixed = False AND Column.Fixed = True Then Return False
	End If
		
	mHeaderColumns.Add(Column)
	
	Return True
End Sub
#End Region

#Region AddDataRow(Row as int, Column as int) as boolean
Public Sub AddDataRow(Row As Int, Column As Int, ShowData As String) As Boolean

       Dim UseThisRow  As TGridDataRow
	   Dim ThisColData As String
	   
	   UseThisRow.Initialize
	   UseThisRow.Columns.Initialize
	   
       
       If mDataRows.Size > 0 Then
	      If Row >= mDataRows.Size Then 
		     UseThisRow.Columns.Add(ShowData)
			 
			 mDataRows.Add(UseThisRow)
			 Return True
		  End If

          UseThisRow = mDataRows.Get(Row)
	   End If
	   
	   If UseThisRow.Columns.Size > 0 Then
	      If Column >= UseThisRow.Columns.Size Then 
		     UseThisRow.Columns.Add(ShowData)
		  Else	  
	         UseThisRow.Columns.Set(Column, ShowData)
		  End If
		  
		  mDataRows.Set(Row, UseThisRow)
		  
		  Return True
		End If
		
		UseThisRow.Columns.Add(ShowData)
		Return True
End Sub
#End Region

#Region Private Functions  CalcFixedColumns / FixedColumnsWidth / HeaderColumnsWidth 
#Region CalcFixedColumns
Private Sub CalcFixedColumns() As Int

        Dim FixedCols As Int = 0
		
		For CurCol = 0 To mHeaderColumns.Size-1	
		    Dim Column As TGridHeaderColumn = mHeaderColumns.Get(CurCol)
			
		    If Column.Fixed = False Then Return FixedCols
			
			FixedCols = FixedCols + 1
		Next
		
		Return FixedCols
End Sub			
#End Region

#Region FixedColumnsWidth
Private Sub FixedColumnsWidth() As Int

        Dim CurCol        As Int
		Dim FixedWidth    As Int    =  (mGridSize + mGridSpacing)
		
		
		For CurCol = 0 To mFixedCols-1
		    If CurCol >= mHeaderColumns.Size Then Exit
			
		    Dim Column As TGridHeaderColumn = mHeaderColumns.Get(CurCol)
			
			
			FixedWidth = FixedWidth + Column.Width + (mGridSize + mGridSpacing)
		Next
		
		Return FixedWidth
End Sub			
#End Region

#Region HeaderColumnsWidth
Private Sub HeaderColumnsWidth() As Int

        Dim CurCol   As Int
		Dim Width    As Int    = (mGridSize + mGridSpacing)
		
		
		For CurCol = mFixedCols To mHeaderColumns.Size-1	
		    Dim Column As TGridHeaderColumn = mHeaderColumns.Get(CurCol)
		
			Width = Width + Column.Width + (mGridSize + mGridSpacing)
		Next
		
		Return Width
End Sub			
#End Region
#End Region

#Region cGrid_Frame Touch / Click / LongClick
'Sub cGrid_Frame_Touch(Action As Int, X As Float, Y As Float)	
'    Dim SentFrom  As Panel = Sender
'
'    #If _Debug Then 
'	    Log("cGrid_Frame_Touch from " &SentFrom.Tag &"  Action:" &Action&"  X:" &X &"  Y:"&Y)
'    #end if				
'End Sub
'Sub cGrid_Frame_Click
'    Dim SentFrom  As Panel = Sender
'
'    #If _Debug Then 
'	    Log("cGrid_Frame_Click from " &SentFrom.Tag)
'    #end if		
'End Sub
'Sub cGrid_Frame_LongClick	
'    Dim SentFrom  As Panel = Sender
'
'    #If _Debug Then 
'	    Log("cGrid_Frame_LongClick from " &SentFrom.Tag)
'    #end if			
'End Sub
#End Region

#Region Overlay_Touch
Sub cGrid_Overlay_Panel_Touch(Action As Int, X As Float, Y As Float)	

    Private ACTION_UP = 1, ACTION_MOVE = 2, ACTION_DOWN = 0 As Int
	
    Dim SentFrom  As Panel = Sender

'    If Action = ACTION_MOVE Then Return


'    #If _Debug Then 
'        Log("cGrid_Overlay_Panel_Touch  Action:" &Action&"  X:" &Floor(X) &" Y:" &Floor(Y) &"X(" &X &") Y(" &Y &")")
'    #end if				
	
	
    If Action = ACTION_DOWN Then	
	   mOverlayTouchX = X
	   mOverlayTouchY = Y
	   Return
	End If


	Dim i       As Int
	
	Dim PosX    As Int     = X
	Dim PosY    As Int     = Y
	
	Dim CurX    As Int     = mGrid_FixedHeader_HSV.ScrollPosition
	Dim CurY    As Int     = mGrid_FixedColumns_VSV.ScrollPosition

    #If _Debug Then 	
	    Dim Hit     As Boolean = False
	#End If 

    #If _Debug Then 
	    Log("cGrid_Overlay_Panel_Touch  CurX:" &CurX &" CurY:" &CurY)
    #end if				
	   
	   
	If mOverlayTouchX <> PosX  Then 
	   If mOverlayTouchX > PosX Then 
	      CurX = CurX + (mOverlayTouchX - PosX)
	   Else
	      CurX = CurX - (PosX - mOverlayTouchX)		  
	   End If
		  
	   If CurX < 0 Then CurX = 0

       #If _Debug Then
    	   Log("cGrid_Overlay_Panel_Touch  CurX:" &CurX &" CurY:" &CurY)
	   #End If
	
	   #If _Debug Then
	       Hit = False
	   #End If
	   
	   For i = 0 To mSnapToGridPositionsH.Length-1
	       #If _Debug Then
	           Log("Compare CurX:" &CurX &" to SnapH:" &mSnapToGridPositionsH(i))
		   #End If
		   
	       If mSnapToGridPositionsH(i) = CurX Then 
		      #If _Debug Then
		          Hit = True
			  #End If
			  
		      Exit
		   End If
		   
		   If mSnapToGridPositionsH(i) > CurX Then 
		      #If _Debug Then
		          Hit  = True
			  #End If 
			  
		      CurX = mSnapToGridPositionsH(i)
			  Exit
		   End If
	   Next
		  	
       #If _Debug Then				
           If Hit = False Then Log("No Hit for CurX:" &CurX)
	   #End If
	   
	   mGrid_FixedHeader_HSV.ScrollPosition     = CurX
	   mGrid_Body_SV2D.HorizontalScrollPosition = CurX
    End If
	   
	If mOverlayTouchY <> PosY  Then 
	   If mOverlayTouchY > PosY Then 
	      CurY = CurY + (mOverlayTouchY - PosY)
	   Else
	      CurY = CurY - (PosY - mOverlayTouchY)		  
	   End If
		  
	   If CurY < 0 Then CurY = 0

       #If _Debug Then
    	   Log("cGrid_Overlay_Panel_Touch  CurY:" &CurY &" CurY:" &CurY)
	   #End If
	   
	   #If _Debug Then
	       Hit = False
	   #End If
	   
	   For i = 0 To mSnapToGridPositionsV.Length-1
	       #If _Debug Then
	           Log("Compare CurY:" &CurY &" to SnapV:" &mSnapToGridPositionsV(i))
		   #End If
	   
	       If mSnapToGridPositionsV(i) = CurY Then 
		      #If _Debug Then
		          Hit  = True
			  #End If 
		   
		      Exit
		   End If
		   
		   If mSnapToGridPositionsV(i) > CurY Then 
		      #If _Debug Then
		          Hit  = True
			  #End If 
		   
		      CurY = mSnapToGridPositionsV(i)
			  Exit
		   End If
	   Next	   

       #If _Debug Then
           If Hit = False Then Log("No Hit for CurY:" &CurY)
	   #End If
		  
       mGrid_FixedColumns_VSV.ScrollPosition    = CurY
	   mGrid_Body_SV2D.VerticalScrollPosition   = CurY
    End If				
End Sub

#Region HorizontalScrollView_ScrollChanged / ScrollView_ScrollChanged / ScrollView2D_ScrollChanged
Sub cGrid_FixedHeader_HSV_ScrollChanged(Position As Int)
Return
    If mInScrollChange Then Return
	
	mLastScrollChange(0) = Position
	mLastScrollChange(1) = -1
	mScrollTimer.Enabled = True	
	Return
'	
'    If mInScrollChange Then Return
'
''    Do While mGrid_Body_SV2D.ScrollingIsFinished = False
''	   DoEvents
''    Loop
'
'
'	#If _Debug Then 
'	    Log("Horz ScrollChange:" &Position)
'    #end if				
'
'	mInScrollChange = True	
'
''	mGrid_Body_SV2D.Enabled                   = False	
'    Log("   Was:" &mGrid_Body_SV2D.HorizontalScrollPosition)
'	
'	If mGrid_Body_SV2D.HorizontalScrollPosition <> Position Then mGrid_Body_SV2D.HorizontalScrollPosition = Position		
''	mGrid_Body_SV2D.Enabled                   = True	
''	DoEvents
'	
'	mInScrollChange = False	
End Sub

Sub cGrid_FixedColumns_VSV_ScrollChanged(Position As Int)
Return
    If mInScrollChange Then Return

	mLastScrollChange(0) = -1
	mLastScrollChange(1) = Position
	mScrollTimer.Enabled = True
	Return
'	
'    If mInScrollChange Then Return
'
''    Do While mGrid_Body_SV2D.ScrollingIsFinished = False
''	   DoEvents
''    Loop
'	
'	#If _Debug Then 
'	    Log("Vert ScrollChange:" &Position)
'    #end if				
'	
'	mInScrollChange = True
'
''	mGrid_Body_SV2D.Enabled                = False
'    If mGrid_Body_SV2D.VerticalScrollPosition <> Position Then mGrid_Body_SV2D.VerticalScrollPosition = Position
''	mGrid_Body_SV2D.SmoothScrollTo(mGrid_FixedHeader_HSV.ScrollPosition, Position)
''	mGrid_Body_SV2D.Enabled                = True
'		
''    DoEvents		
'	
'	mInScrollChange = False		
End Sub

Sub cGrid_Body_SV2D_ScrollChanged(PosX As Int, PosY As Int)
Return
    If mInScrollChange Then Return
	
	mLastScrollChange(0) = PosX
	mLastScrollChange(1) = PosY
	mScrollTimer.Enabled = True
	Return
'	
'    If mInScrollChange Then Return
'	
''    Do While mGrid_Body_SV2D.ScrollingIsFinished = False
''	   DoEvents
''    Loop
'	
'    Log("Both ScrollChange:" &PosX & " / " & PosY)
'	
'	mInScrollChange = True	
'
''	mGrid_FixedHeader_HSV.Enabled         = False
''	mGrid_FixedColumns_VSV.Enabled        = False
'	
''   mGrid_FixedHeader_HSV.ScrollToNow(PosX)
'    Log("    HSV:" &mGrid_FixedHeader_HSV.ScrollPosition &"  VSV:" &mGrid_FixedColumns_VSV.ScrollPosition)
'	mGrid_FixedHeader_HSV.ScrollPosition  = PosX
'	mGrid_FixedColumns_VSV.ScrollPosition = PosY
''	mGrid_FixedColumns_VSV.ScrollToNow(PosY)
'	
''	mGrid_FixedHeader_HSV.Enabled         = True
''	mGrid_FixedColumns_VSV.Enabled        = True	
''   DoEvents	
'	
'	mInScrollChange = False		
End Sub
#End Region

Sub cGrid_ScrollTimer_Tick
Return
    mScrollTimer.Enabled = False
	
    mInScrollChange      = True

    Dim i         As Int
	
	#If _Debug Then
	    Dim HitFound  As Boolean
	    Dim UseHigher As Boolean
    #End If		
	
	If mLastScrollChange(0) <> -1 Then	   
	   #If _Debug Then
	       HitFound  = False
	       UseHigher = False
	   #End If
	   
	   For i = 0 To mSnapToGridPositionsH.Length-1
	       If mSnapToGridPositionsH(i) = mLastScrollChange(0) Then 
		      #If _Debug
		          HitFound = True
		      #End If
		      Exit
		   End If
		   
		   If mSnapToGridPositionsH(i) > mLastScrollChange(0) Then 
		      mLastScrollChange(0) = mSnapToGridPositionsH(i)
			  #If _Debug
			      HitFound  = True
			      UseHigher = True
			  #End If
			  Exit
		   End If
	   Next
	   
	   #If _Debug Then
	       If HitFound = False Then
	          Log("HPos:" &mLastScrollChange(0))
		  
              For i = 0 To mSnapToGridPositionsH.Length-1
    		      Log("Entry:" &i &"  Value:" &mSnapToGridPositionsH(i))
	    	  Next
	       End If
	   #end if
	   
	
	   mGrid_FixedHeader_HSV.ScrollPosition     = mLastScrollChange(0)
	   mGrid_Body_SV2D.HorizontalScrollPosition = mLastScrollChange(0)
	   mGrid_Body_SV2D.GiveFocusToFirstVisible	   
	End If
	
	If mLastScrollChange(1) <> -1 Then
	   #If _Debug Then
	       HitFound  = False
	       UseHigher = False
	   #End If If
	
	   For i = 0 To mSnapToGridPositionsV.Length-1
	       If mSnapToGridPositionsV(i) = mLastScrollChange(1) Then 
		      #If _Debug Then
                  HitFound = True
		      #End If
		      Exit
		   End If
		   
		   If mSnapToGridPositionsV(i) > mLastScrollChange(1) Then 
		      mLastScrollChange(1) = mSnapToGridPositionsV(i)
			  #If _Debug Then
			      HitFound  = True
			      UseHigher = True
		      #End If
			  Exit
		   End If
	   Next
	   
	   #If _Debug Then
    	   If HitFound = False Then
	          Log("VPos:" &mLastScrollChange(1))
		  
              For i = 0 To mSnapToGridPositionsH.Length-1
		          Log("Entry:" &i &"  Value:" &mSnapToGridPositionsV(i))
		      Next
	       End If
	   #End If
	
	   mGrid_FixedColumns_VSV.ScrollPosition    = mLastScrollChange(1)
	   mGrid_Body_SV2D.VerticalScrollPosition   = mLastScrollChange(1)
	   mGrid_Body_SV2D.GiveFocusToFirstVisible	   
	End If
	
	mLastScrollChange(0) = -1
	mLastScrollChange(1) = -1   
	
	mInScrollChange      = False
	
End Sub